
#include <stdio.h>
#include <stdlib.h>

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>


typedef struct led
{
	int pin[2];
	int port[2];
	int light;
}led_t;

led_t matrix[9][9];

int move_no = 0;
int rows[2][3] = {{0}};
int cols[2][3] = {{0}};
int diag[2][2] = {{0}};
int finished = 0;
int first_led = 0;
int playbot = 0;

void init()
{
	DDRA = 0xff;
	DDRB = 0xff;
	DDRC = 0xff;
	DDRD = 0xff;
	
	/* seteaza butoanele ca input si activeaza rezistentele de pull-up */
	
	//buton stanga
	DDRD &= ~(1 << PD3);
	PORTD |= (1 << PD3);
	
	//buton dreapta
	DDRD &= ~(1 << PD5);
	PORTD |= (1 << PD5);
	
	//buton select
	DDRD &= ~(1 << PD4);
	PORTD |= (1 << PD4);
	
	/*seteaza ledurile ca output*/
	DDRA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7);
	DDRC |= (1 << PC1) | (1 << PC2) | (1 << PC3) | (1 << PC4) | (1 << PC5) | (1 << PC6) | (1 << PC7);
	DDRD |= (1 << PD0) | (1 << PD1) | (1 << PD2);
}



/*initializeaza matricea de leduri*/
void init_board()
{
	matrix[0][0].pin[0] = PA2;
	matrix[0][0].pin[1] = PA3;
	
	matrix[0][0].port[0] = 1;
	matrix[0][0].port[1] = 1;
	
	matrix[0][0].light = 0;
	
	matrix[1][0].pin[0] = PA1;
	matrix[1][0].pin[1] = PA4;
	
	matrix[1][0].port[0] = 1;
	matrix[1][0].port[1] = 1;
	
	matrix[1][0].light = 0;
	
	matrix[2][0].pin[0] = PA0;
	matrix[2][0].pin[1] = PA5;
	
	matrix[2][0].port[0] = 1;
	matrix[2][0].port[1] = 1;
	
	matrix[2][0].light = 0;
	
	matrix[0][1].pin[0] = PD0;
	matrix[0][1].pin[1] = PA6;
	
	matrix[0][1].port[0] = 3;
	matrix[0][1].port[1] = 1;
	
	matrix[0][1].light = 0;
	
	matrix[1][1].pin[0] = PD1;
	matrix[1][1].pin[1] = PA7;
	
	matrix[1][1].port[0] = 3;
	matrix[1][1].port[1] = 1;
	
	matrix[1][1].light = 0;
	
	matrix[2][1].pin[0] = PD2;
	matrix[2][1].pin[1] = PC7;
	
	matrix[2][1].port[0] = 3;
	matrix[2][1].port[1] = 2;
	
	matrix[2][1].light = 0;
	
	matrix[0][2].pin[0] = PC4;
	matrix[0][2].pin[1] = PC3;
	
	matrix[0][2].port[0] = 2;
	matrix[0][2].port[1] = 2;
	
	matrix[0][2].light = 0;
	
	matrix[1][2].pin[0] = PC5;
	matrix[1][2].pin[1] = PC2;
	
	matrix[1][2].port[0] = 2;
	matrix[1][2].port[1] = 2;
	
	matrix[1][2].light = 0;
	
	matrix[2][2].pin[0] = PC6;
	matrix[2][2].pin[1] = PC1;
	
	matrix[2][2].port[0] = 2;
	matrix[2][2].port[1] = 2;
	
	matrix[2][2].light = 0;
	
}





/*reseteaza toate ledurile*/
void reset()
{
	int i, j;
	for(i = 0; i < 3; i++)
		for(j = 0; j < 3; j++)
		{
			if(matrix[i][j].port[0] == 1)
			{
				PORTA &= ~(1 << matrix[i][j].pin[0]);
			}
		
			if(matrix[i][j].port[0] == 2)
			{
				PORTC &= ~(1 << matrix[i][j].pin[0]);
			}
			
			if(matrix[i][j].port[0] == 3)
			{
				PORTD &= ~(1 << matrix[i][j].pin[0]);
			}
			if(matrix[i][j].port[1] == 1)
			{
				PORTA &= ~(1 << matrix[i][j].pin[1]);
			}
		
			if(matrix[i][j].port[1] == 2)
			{
				PORTC &= ~(1 << matrix[i][j].pin[1]);
			}
			
			if(matrix[i][j].port[1] == 3)
			{
				PORTD &= ~(1 << matrix[i][j].pin[1]);
			}
		}
}


/*aprinde ledurile de culoarea castigatorului*/
void won(int player)
{
	int i, j;
	_delay_ms(500);
	for(i = 0; i < 3; i++)
		for(j = 0; j < 3; j++)
		{
		
			if(matrix[i][j].port[player] == 1)
			{
				PORTA ^= (1 << matrix[i][j].pin[player]);
				_delay_ms(500);
				PORTA ^= (1 << matrix[i][j].pin[player]);
			}
				
			if(matrix[i][j].port[player] == 2)
			{
				PORTC ^= (1 << matrix[i][j].pin[player]);
				_delay_ms(500);
				PORTC ^= (1 << matrix[i][j].pin[player]);
			}
			
			if(matrix[i][j].port[player] == 3)
			{
				PORTD ^= (1 << matrix[i][j].pin[player]);
				_delay_ms(500);
				PORTD ^= (1 << matrix[i][j].pin[player]);
			}
		}		
}

/*aprinde ledurile indicand remiza*/
void game_over()
{
	int i,j;
	/*se aprind toate ledurile in ambele culori*/
	for(i = 0; i < 3; i++)
		for(j = 0; j < 3; j++)
		{
			if(matrix[i][j].port[0] == 1)
			{
				PORTA ^= (1 << matrix[i][j].pin[0]);
				_delay_ms(500);
				PORTA ^= (1 << matrix[i][j].pin[0]);
			}
			
			if(matrix[i][j].port[0] == 2)
			{
				PORTC ^= (1 << matrix[i][j].pin[0]);
				_delay_ms(500);
				PORTC ^= (1 << matrix[i][j].pin[0]);
			}
				
			if(matrix[i][j].port[0] == 3)
			{
				PORTD ^= (1 << matrix[i][j].pin[0]);
				_delay_ms(500);
				PORTD ^= (1 << matrix[i][j].pin[0]);
				
			}
			
			if(matrix[i][j].port[1] == 1)
			{
				PORTA ^= (1 << matrix[i][j].pin[1]);
				_delay_ms(500);
				PORTA ^= (1 << matrix[i][j].pin[1]);
			}
			else
			if(matrix[i][j].port[1] == 2)
			{
				PORTC ^= (1 << matrix[i][j].pin[1]);
				_delay_ms(500);
				PORTC ^= (1 << matrix[i][j].pin[1]);
			}
			else
			if(matrix[i][j].port[1] == 3)
			{
				PORTD ^= (1 << matrix[i][j].pin[1]);
				_delay_ms(500);
				PORTD ^= (1 << matrix[i][j].pin[1]);
			}
		}

	
}

/*verifica daca un jucator a castigat*/
int verify_finish(int player)
{
	/*daca pe linie, pe coloana sau pe diagonala se afla trei leduri de aceeasi culoare*/
	if(rows[player][0] == 3 || rows[player][1] == 3 || rows[player][2] == 3 ||
		cols[player][0] == 3 || cols[player][1] == 3 || cols[player][2] == 3 ||
		diag[player][0] == 3 || diag[player][1] == 3)
	{	
		//marcheaza terminarea jocului
		finished = 1;
		_delay_ms(1000);
		//stinge toate ledurile
		reset();
		//anunta castigatorul prin aprinderea ledurilor de culoare verde
		won(player);
		return 1;
	}
	return 0;
}


/*aprinde un led*/
int make_move(int row, int col, int player)
{
	//verifica daca led-ul este aprins
	if(matrix[row][col].light != 0)
		return 0;
	
	//incrementeaza numarul de mutari facute
	move_no++;

	//incrementeaza numarul de leduri verzi 
	//aprinse pe linie, coloana, si/sau diagonala
	rows[player][row]++;
	cols[player][col]++;
	if(row == col)
		diag[player][0]++;
	if(row + col == 2)
		diag[player][1]++;
		
	if(matrix[row][col].port[player] == 1)
	{
		PORTA |= (1 << matrix[row][col].pin[player]);
		matrix[row][col].light = 1;
		return 1;
	}
	
	if(matrix[row][col].port[player] == 2)
	{
		PORTC |= (1 << matrix[row][col].pin[player]);
		matrix[row][col].light = 1;
		return 1;
	}
	
	if(matrix[row][col].port[player] == 3)
	{
		PORTD |= (1 << matrix[row][col].pin[player]);
		matrix[row][col].light = 1;
		return 1;
	}
	return 0;

}


/*intoarce primului led stins din stanga
ledului dat ca parametru*/
int move_left(led)
{
	int pos = led-1;
	while(pos >= 0)
	{
		if(matrix[pos/3][pos%3].light == 0)
			return pos;
		pos--;
	}
	return -1;
}


/*intoarce primului led stins din stanga
ledului dat ca parametru*/
int move_right(led)
{
	int pos = led+1;
	while(pos < 9)
	{
		if(matrix[pos/3][pos%3].light == 0)
			return pos;
		pos++;
	}
	return -1;
}






/*reseteaza tot jocul*/
void reset_all()
{
	int i;
	reset();
	finished = 1;
	for(i = 0; i < 3; i++)
	{
		rows[0][i] = 0;
		rows[1][i] = 0;
		cols[0][i] = 0;
		cols[1][i] = 0;
	}
	for(i = 0; i < 2; i++)
	{
		diag[0][i] = 0;
		diag[1][i] = 0;
	}
	for(i = 0; i < 9; i++)
		matrix[i/3][i%3].light = 0;
	first_led = 0;
	move_no = 0;
	playbot = 0;
}


void player_move(player)
{
	int led = first_led;
	int err;
	while(1)
	{
		/*aprinde si stinge ledul curent*/
		if(matrix[led/3][led%3].port[player] == 1)
		{
			PORTA ^= (1 << matrix[led/3][led%3].pin[player]);
			_delay_ms(500);
			PORTA ^= (1 << matrix[led/3][led%3].pin[player]);
			_delay_ms(500);
			
		}
		else
		if(matrix[led/3][led%3].port[player] == 2)
		{
			PORTC ^= (1 << matrix[led/3][led%3].pin[player]);
			_delay_ms(500);
			PORTC ^= (1 << matrix[led/3][led%3].pin[player]);
			_delay_ms(500);
		}
		else
		if(matrix[led%3][led%3].port[player] == 3)
		{
			PORTD ^= (1 << matrix[led/3][led%3].pin[player]);
			_delay_ms(500);
			PORTD ^= (1 << matrix[led/3][led%3].pin[player]);
			_delay_ms(500);
			
		}
		//buton stanga - se muta pe primul led liber din stanga
		if(!(PIND & (1 << PD3)))
		{
			//buton stanga + buton dreapta = resetare joc
			if(!(PIND & (1 << PD5)))
			{
				reset_all();
				return;
			}
			else
			{
				int aux = move_left(led);
				if(aux != -1)
					led = aux;
				continue;
			}
		}
		
		//buton dreapta - se muta pe primul led liber din dreapta
		if(!(PIND & (1 << PD5)))
		{
			//buton stanga + buton dreapta = resetare joc
			if(!(PIND & (1 << PD3)))
			{
				reset_all();
				return;
			}
			else
			{
				int aux = move_right(led);
				if(aux != -1)
					led = aux;
				continue;
			}
		}
		
		//buton mijloc (select) aprinde ledul curent
		if(!(PIND & (1 << PD4)))
		{
			err = make_move(led/3, led%3, player);
			if(err == 1)
			{
				//verifica daca jucatorul a castigat
				verify_finish(player);
				
				//daca a fost selectat primul led liber din matrice
				//primul led devine urmatorul led liber
				if(led == first_led && !finished)
				{
					if(move_left(led) != -1)
						first_led = move_left(led);
					else if (move_right(led) != -1)
							first_led = move_right(led);
						else
						{	//daca nu mai exista leduri libere => remiza
							finished = 1;
							reset();
							game_over();
						}
					
				}
				break;
			}
			
		}
	}
}

//gaseste pozitia ledului liber de pe linia row
int find_row(int row)
{
	int i;
	for(i = 0; i < 3; i++)
		if(matrix[row][i].light == 0)
			return i;
	return -1;
}

//gaseste pozitia ledului liber de pe coloana col
int find_col(int col)
{
	int i;
	for(i = 0; i < 3; i++)
		if(matrix[i][col].light == 0)
			return i;
	return -1;
}

//gaseste pozitia elementului liber de pe diagonala diag
int find_diag(int diag)
{
	int i;
	for(i = 0; i < 3; i++)
		if(diag == 0)
		{
			if(matrix[i][i].light == 0)
				return i;
		}
		else
		{
			if(matrix[i][2-i].light == 0)
				return i;
		}
	return -1;
}


void bot()
{
	int pos, i, led = 0;
	int move = 0;
	
	for(i = 0; i < 3; i++)
	{
		//exista doua leduri rosii aprinse pe aceeasi linie
		if(rows[0][i] == 2 && rows[1][0] == 0)
		{
			pos = find_row(i);
			make_move(i,pos,0);
			led = i*3+pos;
			move = 1;
			break;
		}
		//exista doua leduri rosii aprinse pe aceeasi coloana
		if(cols[0][i] == 2 && cols[1][i] == 0)
		{
			pos = find_col(i);
			make_move(pos,i,0);
			led = pos * 3 + i;
			move = 1;
			break;
		}
	}
	//exista doua leduri rosi aprinse pe aceeasi diagonala
	if((diag[0][0] == 2 && diag[1][0] == 0) && move != 1)
	{
		pos = find_diag(i);
		make_move(pos,pos,0);
		led = pos * 3 + pos;
		move = 1;
	}
	if((diag[0][1] == 2 && diag[1][1] == 0) && move != 1)
	{
		pos = find_diag(i);
		make_move(pos,2-pos,0);
		led = pos * 3 + 2-pos;
		move = 1;
	}
	
	//verifica daca adversarul poate sa catige
	if(!move)	
	{	for(i = 0; i < 3; i++)
		{
			if(rows[1][i] == 2 && rows[0][0] == 0)
			{
				pos = find_row(i);
				make_move(i,pos,0);
				led = i*3+pos;
				move = 1;
				break;
			}
			if(cols[1][i] == 2 && cols[0][i] == 0)
			{
				pos = find_col(i);
				make_move(pos,i,0);
				led = pos * 3 + i;
				move = 1;
				break;
			}
		}
	}
	if((diag[1][0] == 2 && diag[0][0] == 0) && move != 1)
	{
		pos = find_diag(i);
		make_move(pos,pos,0);
		led = pos * 3 + pos;
		move = 1;
	}
	if((diag[1][1] == 2 && diag[0][1] == 0) && move != 1)
	{
		pos = find_diag(i);
		make_move(pos,2-pos,0);
		led = pos * 3 + 2-pos;
		move = 1;
	}
	
	//aprinde led curent
	if(move == 0)
		make_move(first_led/3,first_led%3,0);

	//verifica daca bot-ul e castigator
	verify_finish(0);
	if(led == first_led && !finished)
	{
		if(move_left(led) != -1)
			first_led = move_left(led);
		else if (move_right(led) != -1)
			first_led = move_right(led);
			else
			{//remiza
				finished = 1;
				reset();
				game_over();
			}
					
	}
}


void play_game()
{
	
	while(1)
	{
		//primul jucator (X)
		if(!finished)
			if(playbot == 0)
				player_move(0);
			else 
				bot();
		else return;
		
		//al doilea jucator (0)
		if(!finished)
			player_move(1);
		else return;
	}
}



int main()
{
	init();
	init_board();
	while(1)
	{
		playbot = 0;
		_delay_ms(10000);
		//buton mijloc -> selectare player vs bot
		if(!(PIND & (1 << PD4)))
		{
			playbot = 1;
		}
		play_game();
		reset_all();
		finished = 0;
	}
	
	return 0;
}